<?php

/**
 * This file is automatically generated. Use 'arc liberate' to rebuild it.
 *
 * @generated
 * @phutil-library-version 2
 */
phutil_register_library_map(array(
  '__library_version__' => 2,
  'class' => array(
    'FnClangFormatLinter' => 'lint/linter/FnClangFormatLinter.php',
    'FnCpplintLinter' => 'lint/linter/FnCpplintLinter.php',
    'FnGlslangLinter' => 'lint/linter/FnGlslangLinter.php',
    'FnSequenceMatcher' => 'utils/FnSequenceMatcher.php',
    'FnYAPFLinter' => 'lint/linter/FnYAPFLinter.php',
  ),
  'function' => array(),
  'xmap' => array(
    'FnClangFormatLinter' => 'ArcanistExternalLinter',
    'FnCpplintLinter' => 'ArcanistExternalLinter',
    'FnGlslangLinter' => 'ArcanistExternalLinter',
    'FnSequenceMatcher' => 'Phobject',
    'FnYAPFLinter' => 'ArcanistExternalLinter',
  ),
));
